# -*- coding: utf-8 -*-
"""Various utility functions."""

from chef import autoconfigure, Search
from fabric import api


def mls_config():
    """Get the MLS configuration object for the current node."""
    chef_api = autoconfigure()
    if '127.0.0.1:2222' in api.env.hosts:
        query = 'hostname:%s' % api.env.hostname
    else:
        query = 'ipaddress:%s' % api.env.host
    for node in Search('node', query, api=chef_api):
        try:
            return node.object['mls']
        except KeyError:
            return {}



# -*- coding: utf-8 -*-
"""Bootstrap new servers."""

from fabric import api
from propertyshelf.fabfile.common import bootstrap, rackspace
from propertyshelf.fabfile.common.exceptions import missing_env


@api.task
def database(nodename=None, image=None, flavor=None):
    """Bootstrap a new standalone database server."""
    bootstrap.database(nodename=nodename, image=image, flavor=flavor)


@api.task
def frontend(nodename=None, image=None, flavor=None):
    """Bootstrap a new standalone frontend server."""
    bootstrap.frontend(nodename=nodename, image=image, flavor=flavor)


@api.task
def worker(nodename=None, image=None, flavor=None):
    """Bootstrap a new standalone application worker."""
    bootstrap.worker(nodename=nodename, image=image, flavor=flavor)


@api.task
def bundle_db_fe_app(nodename=None, image=None, flavor=None):
    """Bootstrap a new MLS bundle: Database, Frontend, App Worker."""
    nodename = nodename or api.env.get('nodename_database')
    nodename = nodename or missing_env('nodename_database')

    image = image or api.env.get('os_image')
    image = image or missing_env('os_image')

    flavor = flavor or api.env.get('flavor_database')
    flavor = flavor or missing_env('flavor_database')

    role_database = api.env.get('role_database')
    role_database = role_database or missing_env('role_database')
    role_frontend = api.env.get('role_frontend')
    role_frontend = role_frontend or missing_env('role_frontend')
    role_worker = api.env.get('role_worker')
    role_worker = role_worker or missing_env('role_worker')

    runlist = ','.join([
        'role[%s]' % role_database,
        'role[%s]' % role_frontend,
        'role[%s]' % role_worker,
        'recipe[propertyshelf::rackspace_backup]',
    ])

    opts = dict(
        environment='production',
        flavor=flavor,
        image=image,
        nodename=nodename,
        runlist=runlist,
        servername=nodename,
    )
    rackspace.create(**opts)


@api.task
def staging(nodename=None, image=None, flavor=None):
    """Bootstrap a staging system."""
    bootstrap.staging(nodename=nodename, image=image, flavor=flavor)



# -*- coding: utf-8 -*-
"""Manage chef roles."""

from chef import autoconfigure, Role
from fabric import api
from fabric.colors import green
from propertyshelf.fabfile.common import roles


@api.task
def check():
    """Check if the required roles are available."""
    roles.check_required()


@api.task
def create_missing():
    """Create missing roles on the chef server."""
    chef_api = autoconfigure()
    required = roles.get_required()
    domain = api.env.get('domain', 'example.com')

    # Create the role_database.
    if not roles.check(required.get('role_database')):
        name = required.get('role_database')
        description = 'ZEO Server for %s.' % domain
        run_list = (
            "role[mls_zeoserver]",
        )
        default_attributes = {
            'domain': domain,
        }

        Role.create(
            name,
            api=chef_api,
            description=description,
            run_list=run_list,
            default_attributes=default_attributes,
        )
        print(green('Created role %s') % name)

    # Create the role_frontend.
    if not roles.check(required.get('role_frontend')):
        name = required.get('role_frontend')
        description = 'Frontend Server for %s.' % domain
        run_list = (
            "role[mls_loadbalancer]",
        )
        default_attributes = {
            'domain': domain,
            'haproxy': {
                'app_server_role': required.get('role_worker'),
            },
            'mls': {
                'domain': '.'.join(['mls', domain]),
            },
        }

        Role.create(
            name,
            api=chef_api,
            description=description,
            run_list=run_list,
            default_attributes=default_attributes,
        )
        print(green('Created role %s') % name)

    # Create the role_worker.
    if not roles.check(required.get('role_worker')):
        name = required.get('role_worker')
        description = 'Application Worker for %s.' % domain
        run_list = (
            "role[mls_application]",
        )
        policy_on = api.env.get('mls_policy_enabled') and 'true' or 'false'
        default_attributes = {
            'domain': domain,
            'mls': {
                'customizations': api.env.get('mls_customizations', []),
                'policy': {
                    'enabled': policy_on,
                    'package': api.env.get('mls_policy_package', ''),
                    'package_url': api.env.get('mls_policy_package_url', ''),
                },
                'zeo_role': required.get('role_database'),
            },
        }

        Role.create(
            name,
            api=chef_api,
            description=description,
            run_list=run_list,
            default_attributes=default_attributes,
        )
        print(green('Created role %s') % name)

    # Create the role_staging.
    if not roles.check(required.get('role_staging')):
        name = required.get('role_staging')
        description = 'Staging system for %s.' % domain
        run_list = (
            "role[%s]" % required.get('role_database'),
            "role[%s]" % required.get('role_worker'),
            "role[%s]" % required.get('role_frontend'),
        )
        default_attributes = {
            'domain': domain,
            'haproxy': {
                'app_server_role': name,
            },
            'mls': {
                'domain': '.'.join(['staging', domain]),
                'zeo_role': name,
            },
        }

        Role.create(
            name,
            api=chef_api,
            description=description,
            run_list=run_list,
            default_attributes=default_attributes,
        )
        print(green('Created role %s') % name)


@api.task
def list_nodes(role_list=None):
    """List all available nodes with given roles."""
    roles.list_nodes()



# -*- coding: utf-8 -*-
"""Manage MLS application client components."""

from fabric import api
from mls.fabfile.utils import mls_config
from propertyshelf.fabfile.common import rackspace
from propertyshelf.fabfile.common import utils
from propertyshelf.fabfile.common.exceptions import missing_env


@api.task
def remove():
    """Remove an existing MLS application client."""
    role = api.env.get('role_worker')
    role = role or missing_env('role_worker')
    opts = dict(
        environment='production',
        role=role,
    )
    rackspace.remove(**opts)


@api.task
@api.roles('worker')
def update():
    """Update the client packages."""
    utils.supervisorctl(command='stop', service='application')
    utils.backup_dev_packages(config=mls_config())
    utils.run_buildout(config=mls_config())
    utils.supervisorctl(command='start', service='application')


@api.task
@api.roles('worker')
def restart():
    """Restart the application client component."""
    utils.supervisorctl(command='restart', service='application')


@api.task
@api.roles('worker')
def rebuild():
    """Rebuild the application using buildout."""
    utils.run_buildout(config=mls_config())
    utils.supervisorctl(command='restart', service='application')


@api.task
@api.roles('database')
def update_support_client():
    """Update the maintenance client packages."""
    utils.backup_dev_packages(config=mls_config(), folder='~/maintenance')
    utils.run_buildout(config=mls_config(), folder='~/maintenance')



# -*- coding: utf-8 -*-
"""Manage MLS database components."""

from fabric import api
from mls.fabfile.utils import mls_config
from propertyshelf.fabfile.common import utils, zodb


def zodb_ctl(command=None):
    """ZODB database control for start/stop/restart."""
    utils.supervisorctl(command=command, service='zeoserver')


@api.task
@api.roles('database')
def restart():
    """Restart the database component."""
    zodb_ctl('restart')


@api.task
@api.roles('database')
def download_data():
    """Download the database files from the server."""
    zodb.download_data(mls_config())


@api.task
@api.roles('database')
def download_zodb():
    """Download ZODB part of Zope's data from the server."""
    zodb.download_zodb(mls_config())


@api.task
@api.roles('database')
def download_blobs():
    """Download blob part of Zope's data from the server."""
    zodb.download_blobs(mls_config())


@api.task
@api.roles('database')
def upload_data():
    """Upload the database files to the server."""
    zodb.upload_data(mls_config())
    zodb_ctl('restart')


@api.task
@api.roles('database')
def upload_zodb(start_when_done=True):
    """Upload ZODB part of Zope's data to the server."""
    zodb.upload_zodb(mls_config())
    zodb_ctl('restart')


@api.task
@api.roles('database')
def upload_blob(start_when_done=True):
    """Upload blob part of Zope's data to the server."""
    zodb.upload_blob(mls_config())
    zodb_ctl('restart')


@api.task
@api.roles('database')
def snapshot_backup():
    """Perform a snapshot backup."""
    zodb.backup(mls_config())



# -*- coding: utf-8 -*-
"""Sample MLS deployment script."""

from fabric import api

from mls.fabfile import *
from propertyshelf.fabfile.common.environments import *


# Definition of role names to be used.
api.env.role_database = 'mls_db'
api.env.role_frontend = 'mls_fe'
api.env.role_staging = 'mls_staging'
api.env.role_worker = 'mls_app'

# Definition of used Rackspace flavors (server sized) for our servers.
api.env.flavor_database = '5'
api.env.flavor_frontend = '2'
api.env.flavor_staging = '2'
api.env.flavor_worker = '3'

# Definition of node names to be used.
api.env.nodename_database = 'mls-db'
api.env.nodename_frontend = 'mls-fe'
api.env.nodename_staging = 'mls-staging'
api.env.nodename_worker = 'mls-app'

# The Rackspace server image we use. This is a Debian 6.0.6.
api.env.os_image = '92a28e50-181d-4fc7-a071-567d26fc95f6'

# MLS specific configuration.
api.env.domain = 'mls-example.com'
api.env.mls_customizations = ['mlsext.realtorcom', ]
api.env.mls_policy_enabled = True
api.env.mls_policy_package = 'mlspolicy.example'
api.env.mls_policy_package_url = 'git https://github.com/yourname/mlspolicy.example'
